@@ -66,6 +66,7 @@
 QT_BEGIN_NAMESPACE
 
 #ifndef QT_NO_SYSTEMLOCALE
+struct QLocaleData;
 class Q_CORE_EXPORT QSystemLocale
 {
 public:
@@ -126,6 +127,7 @@ public:
     virtual QVariant query(QueryType type, QVariant in) const;
     virtual QLocale fallbackUiLocale() const;
 
+    inline const QLocaleData *fallbackUiLocaleData() const;
 private:
     QSystemLocale(bool);
     friend class QSystemLocaleSingleton;
@@ -249,35 +251,39 @@ public:
         if (std::fabs(d) > std::numeric_limits<float>::max()) {
             if (ok != 0)
                 *ok = false;
-            return 0.0f;
+            const float huge = std::numeric_limits<float>::infinity();
+            return d < 0 ? -huge : huge;
+        }
+        if (d != 0 && float(d) == 0) {
+            // Values that underflow double already failed. Match them:
+            if (ok != 0)
+                *ok = false;
+            return 0;
         }
         return float(d);
     }
 
-    double stringToDouble(const QChar *begin, int len, bool *ok,
-                          QLocale::NumberOptions number_options) const;
-    qint64 stringToLongLong(const QChar *begin, int len, int base, bool *ok,
-                            QLocale::NumberOptions number_options) const;
-    quint64 stringToUnsLongLong(const QChar *begin, int len, int base, bool *ok,
-                                QLocale::NumberOptions number_options) const;
+    double stringToDouble(QStringView str, bool *ok, QLocale::NumberOptions options) const;
+    qint64 stringToLongLong(QStringView str, int base, bool *ok, QLocale::NumberOptions options) const;
+    quint64 stringToUnsLongLong(QStringView str, int base, bool *ok, QLocale::NumberOptions options) const;
 
-    // these functions are used in QIntValidator (QtGui)
-    Q_CORE_EXPORT static double bytearrayToDouble(const char *num, bool *ok, bool *overflow = 0);
-    Q_CORE_EXPORT static qint64 bytearrayToLongLong(const char *num, int base, bool *ok, bool *overflow = 0);
-    Q_CORE_EXPORT static quint64 bytearrayToUnsLongLong(const char *num, int base, bool *ok);
+    static double bytearrayToDouble(const char *num, bool *ok);
+    // this function is used in QIntValidator (QtGui)
+    Q_CORE_EXPORT static qint64 bytearrayToLongLong(const char *num, int base, bool *ok);
+    static quint64 bytearrayToUnsLongLong(const char *num, int base, bool *ok);
 
-    bool numberToCLocale(const QChar *str, int len, QLocale::NumberOptions number_options,
+    bool numberToCLocale(QStringView s, QLocale::NumberOptions number_options,
                          CharBuff *result) const;
     inline char digitToCLocale(QChar c) const;
 
     // this function is used in QIntValidator (QtGui)
-    Q_CORE_EXPORT bool validateChars(
-            const QString &str, NumberMode numMode, QByteArray *buff, int decDigits = -1,
+    Q_CORE_EXPORT bool validateChars(QStringView str, NumberMode numMode, QByteArray *buff, int decDigits = -1,
             QLocale::NumberOptions number_options = QLocale::DefaultNumberOptions) const;
 
 public:
     quint16 m_language_id, m_script_id, m_country_id;
 
+    // FIXME QTBUG-69324: not all unicode code-points map to single-token UTF-16 :-(
     quint16 m_decimal, m_group, m_list, m_percent, m_zero, m_minus, m_plus, m_exponential;
     quint16 m_quotation_start, m_quotation_end;
     quint16 m_alternate_quotation_start, m_alternate_quotation_end;
@@ -304,6 +310,9 @@ public:
     quint16 m_narrow_day_names_idx, m_narrow_day_names_size;
     quint16 m_am_idx, m_am_size;
     quint16 m_pm_idx, m_pm_size;
+    quint16 m_byte_idx, m_byte_size;
+    quint16 m_byte_si_quantified_idx, m_byte_si_quantified_size;
+    quint16 m_byte_iec_quantified_idx, m_byte_iec_quantified_size;
     char    m_currency_iso_code[3];
     quint16 m_currency_symbol_idx, m_currency_symbol_size;
     quint16 m_currency_display_name_idx, m_currency_display_name_size;
@@ -349,31 +358,22 @@ public:
 
     QByteArray bcp47Name(char separator = '-') const;
 
-    // ### QByteArray::fromRawData would be more optimal
-    inline QString languageCode() const { return QLocalePrivate::languageToCode(QLocale::Language(m_data->m_language_id)); }
-    inline QString scriptCode() const { return QLocalePrivate::scriptToCode(QLocale::Script(m_data->m_script_id)); }
-    inline QString countryCode() const { return QLocalePrivate::countryToCode(QLocale::Country(m_data->m_country_id)); }
+    inline QLatin1String languageCode() const { return QLocalePrivate::languageToCode(QLocale::Language(m_data->m_language_id)); }
+    inline QLatin1String scriptCode() const { return QLocalePrivate::scriptToCode(QLocale::Script(m_data->m_script_id)); }
+    inline QLatin1String countryCode() const { return QLocalePrivate::countryToCode(QLocale::Country(m_data->m_country_id)); }
 
-    static QString languageToCode(QLocale::Language language);
-    static QString scriptToCode(QLocale::Script script);
-    static QString countryToCode(QLocale::Country country);
-    static QLocale::Language codeToLanguage(const QChar *code, int len) Q_DECL_NOTHROW;
-    static QLocale::Language codeToLanguage(const QString &code) Q_DECL_NOTHROW { return codeToLanguage(code.data(), code.size()); }
-    static QLocale::Language codeToLanguage(const QStringRef &code) Q_DECL_NOTHROW { return codeToLanguage(code.data(), code.size()); }
-    static QLocale::Script codeToScript(const QChar *code, int len) Q_DECL_NOTHROW;
-    static QLocale::Script codeToScript(const QString &code) Q_DECL_NOTHROW { return codeToScript(code.data(), code.size()); }
-    static QLocale::Script codeToScript(const QStringRef &code) Q_DECL_NOTHROW { return codeToScript(code.data(), code.size()); }
-    static QLocale::Country codeToCountry(const QChar *code, int len) Q_DECL_NOTHROW;
-    static QLocale::Country codeToCountry(const QString &code) Q_DECL_NOTHROW { return codeToCountry(code.data(), code.size()); }
-    static QLocale::Country codeToCountry(const QStringRef &code) Q_DECL_NOTHROW { return codeToCountry(code.data(), code.size()); }
+    static QLatin1String languageToCode(QLocale::Language language);
+    static QLatin1String scriptToCode(QLocale::Script script);
+    static QLatin1String countryToCode(QLocale::Country country);
+    static QLocale::Language codeToLanguage(QStringView code) Q_DECL_NOTHROW;
+    static QLocale::Script codeToScript(QStringView code) Q_DECL_NOTHROW;
+    static QLocale::Country codeToCountry(QStringView code) Q_DECL_NOTHROW;
     static void getLangAndCountry(const QString &name, QLocale::Language &lang,
                                   QLocale::Script &script, QLocale::Country &cntry);
 
     QLocale::MeasurementSystem measurementSystem() const;
 
-    static void updateSystemPrivate();
-
-    QString dateTimeToString(const QString &format, const QDateTime &datetime,
+    QString dateTimeToString(QStringView format, const QDateTime &datetime,
                              const QDate &dateOnly, const QTime &timeOnly,
                              const QLocale *q) const;
 
@@ -382,6 +382,10 @@ public:
     QLocale::NumberOptions m_numberOptions;
 };
 
+#ifndef QT_NO_SYSTEMLOCALE
+const QLocaleData *QSystemLocale::fallbackUiLocaleData() const { return fallbackUiLocale().d->m_data; }
+#endif
+
 template <>
 inline QLocalePrivate *QSharedDataPointer<QLocalePrivate>::clone()
 {
@@ -415,17 +419,18 @@ inline char QLocaleData::digitToCLocale(QChar in) const
     if (in == m_exponential || in == QChar::toUpper(m_exponential))
         return 'e';
 
-    // In several languages group() is the char 0xA0, which looks like a space.
-    // People use a regular space instead of it and complain it doesn't work.
-    if (m_group == 0xA0 && in.unicode() == ' ')
+    // In several languages group() is a non-breaking space (U+00A0) or its thin
+    // version (U+202f), which look like spaces.  People (and thus some of our
+    // tests) use a regular space instead and complain if it doesn't work.
+    if ((m_group == 0xA0 || m_group == 0x202f) && in.unicode() == ' ')
         return ',';
 
     return 0;
 }
 
-QString qt_readEscapedFormatString(const QString &format, int *idx);
+QString qt_readEscapedFormatString(QStringView format, int *idx);
 bool qt_splitLocaleName(const QString &name, QString &lang, QString &script, QString &cntry);
-int qt_repeatCount(const QString &s, int i);
+int qt_repeatCount(QStringView s);
 
 enum { AsciiSpaceMask = (1u << (' ' - 1)) |
                         (1u << ('\t' - 1)) |   // 9: HT - horizontal tab
