@@ -7,16 +7,22 @@
 #define QT_FEATURE_simulator_and_device -1
 #define QT_FEATURE_build_all -1
 #define QT_FEATURE_c__11 1
-#define QT_FEATURE_c__14 -1
-#define QT_FEATURE_c__1z -1
+#define QT_FEATURE_c__14 1
+#define QT_FEATURE_c__1z 1
+#define QT_FEATURE_c__2a -1
+#define QT_FEATURE_c99 1
+#define QT_FEATURE_c11 1
+#define QT_FEATURE_thread 1
+#define QT_FEATURE_future 1
 #define QT_FEATURE_concurrent 1
 #define QT_FEATURE_pkg_config 1
 #define QT_FEATURE_force_asserts -1
 #define QT_LARGEFILE_SUPPORT 64
+#define QT_COMPILER_SUPPORTS_NEON 1
 #define QT_VISIBILITY_AVAILABLE true
 #define QT_FEATURE_separate_debug_info -1
 #define QT_FEATURE_static -1
-#define QT_VERSION_STR "5.9.7"
+#define QT_VERSION_STR "5.13.2"
 #define QT_VERSION_MAJOR 5
-#define QT_VERSION_MINOR 9
-#define QT_VERSION_PATCH 7
+#define QT_VERSION_MINOR 13
+#define QT_VERSION_PATCH 2
