@@ -4,6 +4,10 @@
 #define QT_FEATURE_big_codecs 1
 #define QT_FEATURE_codecs 1
 #define QT_FEATURE_commandlineparser 1
+#define QT_FEATURE_itemmodel 1
+#define QT_FEATURE_proxymodel 1
+#define QT_FEATURE_concatenatetablesproxymodel 1
+#define QT_FEATURE_cxx11_future 1
 #define QT_FEATURE_textdate 1
 #define QT_FEATURE_datestring 1
 #define QT_NO_EVENTFD 
@@ -11,8 +15,6 @@
 #define QT_FEATURE_filesystemwatcher 1
 #define QT_FEATURE_gestures 1
 #define QT_NO_ICONV 
-#define QT_FEATURE_itemmodel 1
-#define QT_FEATURE_proxymodel 1
 #define QT_FEATURE_identityproxymodel 1
 #define QT_NO_INOTIFY 
 #define QT_FEATURE_library 1
@@ -25,6 +27,7 @@
 #define QT_FEATURE_settings 1
 #define QT_FEATURE_sharedmemory 1
 #define QT_FEATURE_sortfilterproxymodel 1
+#define QT_FEATURE_std_atomic64 1
 #define QT_FEATURE_stringlistmodel 1
 #define QT_FEATURE_systemsemaphore 1
 #define QT_FEATURE_temporaryfile 1
@@ -32,6 +35,7 @@
 #define QT_FEATURE_timezone 1
 #define QT_FEATURE_topleveldomain 1
 #define QT_FEATURE_translation 1
+#define QT_FEATURE_transposeproxymodel 1
 #define QT_FEATURE_xmlstream 1
 #define QT_FEATURE_xmlstreamreader 1
 #define QT_FEATURE_xmlstreamwriter 1
