@@ -1,10 +1,16 @@
 #define QT_FEATURE_networkinterface 1
 #define QT_FEATURE_bearermanagement 1
+#define QT_FEATURE_dnslookup 1
+#define QT_FEATURE_schannel -1
+#define QT_LINKED_OPENSSL true
+#define QT_FEATURE_dtls -1
 #define QT_FEATURE_ftp 1
 #define QT_FEATURE_http 1
 #define QT_FEATURE_localserver 1
 #define QT_FEATURE_networkdiskcache 1
 #define QT_FEATURE_networkproxy 1
+#define QT_FEATURE_opensslv11 -1
+#define QT_FEATURE_ocsp -1
 #define QT_FEATURE_sctp -1
 #define QT_NO_SCTP 
 #define QT_FEATURE_socks5 1
