@@ -1,32 +1,39 @@
 #!/usr/bin/env perl
 #############################################################################
 ##
-## Copyright (C) 2015 The Qt Company Ltd.
-## Contact: http://www.qt.io/licensing/
+## Copyright (C) 2016 The Qt Company Ltd.
+## Copyright (C) 2016 Intel Corporation.
+## Contact: https://www.qt.io/licensing/
 ##
 ## This file is part of the build configuration tools of the Qt Toolkit.
 ##
-## $QT_BEGIN_LICENSE:LGPL21$
+## $QT_BEGIN_LICENSE:LGPL$
 ## Commercial License Usage
 ## Licensees holding valid commercial Qt licenses may use this file in
 ## accordance with the commercial license agreement provided with the
 ## Software or, alternatively, in accordance with the terms contained in
 ## a written agreement between you and The Qt Company. For licensing terms
-## and conditions see http://www.qt.io/terms-conditions. For further
-## information use the contact form at http://www.qt.io/contact-us.
+## and conditions see https://www.qt.io/terms-conditions. For further
+## information use the contact form at https://www.qt.io/contact-us.
 ##
 ## GNU Lesser General Public License Usage
 ## Alternatively, this file may be used under the terms of the GNU Lesser
-## General Public License version 2.1 or version 3 as published by the Free
-## Software Foundation and appearing in the file LICENSE.LGPLv21 and
-## LICENSE.LGPLv3 included in the packaging of this file. Please review the
-## following information to ensure the GNU Lesser General Public License
-## requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-## http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+## General Public License version 3 as published by the Free Software
+## Foundation and appearing in the file LICENSE.LGPL3 included in the
+## packaging of this file. Please review the following information to
+## ensure the GNU Lesser General Public License version 3 requirements
+## will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
 ##
-## As a special exception, The Qt Company gives you certain additional
-## rights. These rights are described in The Qt Company LGPL Exception
-## version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+## GNU General Public License Usage
+## Alternatively, this file may be used under the terms of the GNU
+## General Public License version 2.0 or (at your option) the GNU General
+## Public license version 3 or any later version approved by the KDE Free
+## Qt Foundation. The licenses are as published by the Free Software
+## Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+## included in the packaging of this file. Please review the following
+## information to ensure the GNU General Public License requirements will
+## be met: https://www.gnu.org/licenses/gpl-2.0.html and
+## https://www.gnu.org/licenses/gpl-3.0.html.
 ##
 ## $QT_END_LICENSE$
 ##
@@ -69,15 +76,15 @@ sub normalizePath {
 # set output basedir to be where ever syncqt is run from
 our $out_basedir = getcwd();
 normalizePath(\$out_basedir);
+our $build_basedir;
 our $basedir;
-our $quoted_basedir;
 
 # Make sure we use Windows line endings for chomp and friends on Windows.
 $INPUT_RECORD_SEPARATOR = "\r\n" if ($^O eq "msys");
 
 # will be defined based on the modules sync.profile
 our (%modules, %moduleheaders, @allmoduleheadersprivate, %classnames, %deprecatedheaders);
-our @qpa_headers = ();
+our (@qpa_headers, @private_headers);
 
 # will be derived from sync.profile
 our %reverse_classnames = ();
@@ -91,14 +98,11 @@ my $showonly = 0;
 my $verbose_level = 1;
 my $remove_stale = 1;
 my $force_win = 0;
-my $force_relative = 0;
 my $check_includes = 0;
 my $copy_headers = 0;
-my $create_private_headers = 1;
 my $minimal = 0;
 my $module_version = 0;
 my @modules_to_sync ;
-$force_relative = 1 if ( -d "/System/Library/Frameworks" );
 
 
 # functions ----------------------------------------------------------
@@ -117,11 +121,11 @@ sub showUsage
 
     print "  -copy                 Copy headers instead of include-fwd(default: " . ($copy_headers ? "yes" : "no") . ")\n";
     print "  -remove-stale         Removes stale headers              (default: " . ($remove_stale ? "yes" : "no") . ")\n";
-    print "  -relative             Force relative symlinks            (default: " . ($force_relative ? "yes" : "no") . ")\n";
     print "  -windows              Force platform to Windows          (default: " . ($force_win ? "yes" : "no") . ")\n";
     print "  -showonly             Show action but not perform        (default: " . ($showonly ? "yes" : "no") . ")\n";
     print "  -minimal              Do not create CamelCase headers    (default: " . ($minimal ? "yes" : "no") . ")\n";
     print "  -outdir <PATH>        Specify output directory for sync  (default: $out_basedir)\n";
+    print "  -builddir <PATH>      Specify build directory for sync   (default: same as -outdir)\n";
     print "  -version <VERSION>    Specify the module's version       (default: detect from qglobal.h)\n";
     print "  -quiet                Only report problems, not activity (same as -verbose 0)\n";
     print "  -v, -verbose <level>  Sets the verbosity level (max. 4)  (default: $verbose_level)\n";
@@ -129,7 +133,6 @@ sub showUsage
     print "  -separate-module <NAME>:<PROFILEDIR>:<HEADERDIR>\n";
     print "                        Create headers for <NAME> with original headers in\n";
     print "                        <HEADERDIR> relative to <PROFILEDIR> \n";
-    print "  -private              Force copy private headers         (default: " . ($create_private_headers ? "yes" : "no") . ")\n";
     print "  -help                 This help\n";
     exit 0;
 }
@@ -193,9 +196,11 @@ sub shouldMasterInclude {
 }
 
 ######################################################################
-# Syntax:  classNames(iheader, clean)
+# Syntax:  classNames(iheader, clean, requires)
 # Params:  iheader, string, filename to parse for classname "symlinks"
 #          (out) clean, boolean, will be set to false if the header isn't clean
+#          (out) requires, string, will be set to non-empty if the header
+#                                  requires a feature
 #
 # Purpose: Scans through iheader to find all classnames that should be
 #          synced into library's include structure.
@@ -203,12 +208,11 @@ sub shouldMasterInclude {
 ######################################################################
 sub classNames {
     my @ret;
-    my ($iheader, $clean) = @_;
+    my ($iheader, $clean, $requires) = @_;
     $$clean = 1;
+    $$requires = "";
 
     my $ihdrbase = basename($iheader);
-    my $classname = $classnames{$ihdrbase};
-    push @ret, split(/,/, $classname) if ($classname);
 
     my $parsable = "";
     if(open(F, "<$iheader")) {
@@ -224,11 +228,12 @@ sub classNames {
             }
             if($line) {
                 $line =~ s,//.*$,,; #remove c++ comments
-                $line .= ";" if($line =~ m/^Q_[A-Z_]*\(.*\)[\r\n]*$/); #qt macro
+                $line .= ";" if($line =~ m/^Q_[A-Z_0-9]*\(.*\)[\r\n]*$/); #qt macro
                 $line .= ";" if($line =~ m/^QT_(BEGIN|END)_HEADER[\r\n]*$/); #qt macro
                 $line .= ";" if($line =~ m/^QT_(BEGIN|END)_NAMESPACE(_[A-Z]+)*[\r\n]*$/); #qt macro
                 $line .= ";" if($line =~ m/^QT_MODULE\(.*\)[\r\n]*$/); # QT_MODULE macro
                 $line .= ";" if($line =~ m/^QT_WARNING_(PUSH|POP|DISABLE_\w+\(.*\))[\r\n]*$/); # qt macros
+                $$requires = $1 if ($line =~ m/^QT_REQUIRE_CONFIG\((.*)\);[\r\n]*$/);
                 $parsable .= " " . $line;
             }
         }
@@ -326,8 +331,8 @@ sub check_header {
     my ($lib, $header, $iheader, $public_header, $private_header) = @_;
     my $header_skip_qt_begin_namespace_test = 0;
 
+    return if ($ignore_for_include_check{$header});
     if ($public_header) {
-        return if ($ignore_for_include_check{$header});
         $header_skip_qt_begin_namespace_test = 1 if ($ignore_for_qt_begin_namespace_check{$header});
     }
 
@@ -355,7 +360,7 @@ sub check_header {
                 $include = 0;
             }
             if ($include && $public_header) {
-                print STDERR "$lib: ERROR: $iheader includes private header $include\n" if ($include =~ /_p.h$/);
+                print STDERR "$lib: ERROR: $iheader includes private header $include\n" if ($include =~ /_p\.h$/);
                 for my $trylib (keys(%modules)) {
                     if (-e "$out_basedir/include/$trylib/$include") {
                         print STDERR "$lib: WARNING: $iheader includes $include when it should include $trylib/$include\n";
@@ -582,21 +587,18 @@ sub copyFile
 }
 
 ######################################################################
-# Syntax:  findFiles(dir, match, descend)
+# Syntax:  findFiles(dir, match)
 # Params:  dir, string, directory to search for name
 #          match, string, regular expression to match in dir
-#          descend, integer, 0 = non-recursive search
-#                            1 = recurse search into subdirectories
 #
 # Purpose: Finds files matching a regular expression.
 # Returns: List of matching files.
 #
-# Examples:
-#   findFiles("/usr","\.cpp$",1)  - finds .cpp files in /usr and below
-#   findFiles("/tmp","^#",0)      - finds #* files in /tmp
+# Example:
+#   findFiles("/tmp", "^#")      - finds #* files in /tmp
 ######################################################################
 sub findFiles {
-    my ($dir,$match,$descend) = @_;
+    my ($dir, $match) = @_;
     my ($file,$p,@files);
     local(*D);
     normalizePath(\$dir);
@@ -611,9 +613,6 @@ sub findFiles {
             next if ( $file  =~ /^\.\.?$/ );
             $p = $file;
             ($file =~ /$match/) && (push @files, $p);
-            if ( $descend && -d $p && ! -l $p ) {
-                push @files, &findFiles($p,$match,$descend);
-            }
         }
         closedir(D);
     }
@@ -639,17 +638,16 @@ sub listSubdirs {
 ######################################################################
 # Syntax:  loadSyncProfile()
 #
-# Purpose: Locates the sync.profile.
-# Returns: Hashmap of module name -> directory.
+# Purpose: Loads the sync.profile.
 ######################################################################
 sub loadSyncProfile {
-    my ($srcbase, $outbase) = @_;
     if ($verbose_level) {
-        print("<srcbase> = $$srcbase \n");
-        print("<outbase> = $$outbase \n");
+        print("<srcbase> = $basedir \n");
+        print("<bldbase> = $build_basedir \n");
+        print("<outbase> = $out_basedir \n");
     }
 
-    my $syncprofile = "$$srcbase/sync.profile";
+    my $syncprofile = "$basedir/sync.profile";
     my $result;
     unless ($result = do "$syncprofile") {
         die "syncqt couldn't parse $syncprofile: $@" if $@;
@@ -662,12 +660,13 @@ sub loadSyncProfile {
         }
     }
 
-    return $result;
+    push @private_headers, qr/_p(ch)?\.h$/;
 }
 
 sub basePrettify {
     my ($arg) = @_;
     $$arg =~ s,^\Q$basedir\E,<srcbase>,;
+    $$arg =~ s,^\Q$build_basedir\E,<bldbase>,;
     $$arg =~ s,^\Q$out_basedir\E,<outbase>,;
 }
 
@@ -677,6 +676,11 @@ sub cleanPath {
     return $arg;
 }
 
+######################################################################
+# Syntax:  locateSyncProfile()
+#
+# Purpose: Locates the sync.profile.
+######################################################################
 sub locateSyncProfile
 {
     my ($directory) = @_;
@@ -699,12 +703,35 @@ sub isQpaHeader
     return 0;
 }
 
+sub isPrivateHeader
+{
+    my ($header) = @_;
+    foreach my $private_header (@private_headers) {
+        return 1 if ($header =~ $private_header);
+    }
+    return 0;
+}
+
+sub globosort($$) {
+    my ($a, $b) = @_;
+    if ($a =~ /^q(.*)global\.h$/) {
+        my $sa = $1;
+        if ($b =~ /^q(.*)global\.h$/) {
+            my $sb = $1;
+            # Compare stems so qglobal.h (empty stem) is first:
+            return $sa cmp $sb;
+        }
+        return -1; # $a is global, so before $b
+    }
+    return +1 if $b =~ /^q.*global\.h$/; # $a not global, so after $b
+    return $a cmp $b;
+}
+
 # check if this is an in-source build, and if so use that as the basedir too
 $basedir = locateSyncProfile($out_basedir);
 if ($basedir) {
     $basedir = dirname($basedir) ;
     normalizePath(\$basedir);
-    $quoted_basedir = "\Q$basedir";
 }
 
 # --------------------------------------------------------------------
@@ -726,6 +753,9 @@ while ( @ARGV ) {
     } elsif($arg eq "-o" || $arg eq "-outdir") {
         $var = "output";
         $val = shift @ARGV;
+    } elsif($arg eq "-builddir") {
+        $var = "build";
+        $val = shift @ARGV;
     } elsif($arg eq "-showonly" || $arg eq "-remove-stale" || $arg eq "-windows" ||
             $arg eq "-relative" || $arg eq "-check-includes") {
         $var = substr($arg, 1);
@@ -758,9 +788,6 @@ while ( @ARGV ) {
     } elsif($arg eq "-minimal") {
         $var = "minimal";
         $val = "yes";
-    } elsif($arg eq "-private") {
-        $var = "create_private_headers";
-        $val = "yes";
     } elsif($arg eq "-version") {
         $var = "version";
         $val = shift @ARGV;
@@ -772,7 +799,6 @@ while ( @ARGV ) {
         die "Could not find a sync.profile for '$arg'\n" if (!$basedir);
         $basedir = dirname($basedir);
         normalizePath(\$basedir);
-        $quoted_basedir = "\Q$basedir";
         $var = "ignore";
     }
 
@@ -818,12 +844,6 @@ while ( @ARGV ) {
         } elsif($force_win) {
             $force_win--;
         }
-    } elsif ($var eq "relative") {
-        if($val eq "yes") {
-            $force_relative++;
-        } elsif($force_relative) {
-            $force_relative--;
-        }
     } elsif ($var eq "minimal") {
         if($val eq "yes") {
             $minimal++;
@@ -853,6 +873,16 @@ while ( @ARGV ) {
             $out_basedir = $outdir;
         }
         normalizePath(\$out_basedir);
+    } elsif ($var eq "build") {
+        my $outdir = $val;
+        if (checkRelative($outdir)) {
+            $build_basedir = getcwd();
+            chomp $build_basedir;
+            $build_basedir .= "/" . $outdir;
+        } else {
+            $build_basedir = $outdir;
+        }
+        normalizePath(\$build_basedir);
     }
 }
 
@@ -860,18 +890,23 @@ while ( @ARGV ) {
 die "Could not find any sync.profile for your module!\nPass <module directory> to syncqt to sync your header files.\nsyncqt failed" if (!$basedir);
 die "The -version argument is mandatory" if (!$module_version);
 
+$build_basedir = $out_basedir if (!defined($build_basedir));
+
 our @ignore_headers = ();
-our @ignore_for_master_contents = ();
 our @ignore_for_include_check = ();
 our @ignore_for_qt_begin_namespace_check = ();
 our @ignore_for_qt_module_check = ();
 our %inject_headers = ();
 
 # load the module's sync.profile here, before we can
-loadSyncProfile(\$basedir, \$out_basedir);
+loadSyncProfile();
 
 @modules_to_sync = keys(%modules) if($#modules_to_sync == -1);
 
+for my $p (keys %inject_headers) {
+    push @ignore_for_include_check, @{$inject_headers{$p}};
+}
+
 my %allmoduleheadersprivate = map { $_ => 1 } @allmoduleheadersprivate;
 %ignore_for_include_check = map { $_ => 1 } @ignore_for_include_check;
 %ignore_for_qt_begin_namespace_check = map { $_ => 1 } @ignore_for_qt_begin_namespace_check;
@@ -895,7 +930,7 @@ foreach my $lib (@modules_to_sync) {
     $allheadersprivate = 1 if $allmoduleheadersprivate{$lib};
 
     #information used after the syncing
-    my $pri_install_classes = "";
+    my $pri_install_gfiles = "";
     my $pri_install_files = "";
     my $pri_install_pfiles = "";
     my $pri_install_qpafiles = "";
@@ -903,18 +938,15 @@ foreach my $lib (@modules_to_sync) {
     my $pri_clean_files = "";
 
     my $libcapitals = uc($lib);
-    my $master_contents =
-        "#ifndef QT_".$libcapitals."_MODULE_H\n" .
-        "#define QT_".$libcapitals."_MODULE_H\n" .
-        "#include <$lib/${lib}Depends>\n";
+    my %master_contents = ();
 
     #remove the old files
-    if($remove_stale) {
+    if ($remove_stale && !$minimal) {
         my %injections = ();
         for my $p (keys %inject_headers) {
             next unless ($p =~ /^\Q$dir\E(\/|$)/);
             my $sp = $p;
-            $sp =~ s,^\Q$basedir\E/,$out_basedir/,;
+            $sp =~ s,^\Q$basedir\E/,$build_basedir/,;
             for my $n (@{$inject_headers{$p}}) {
                 $injections{$sp."/".$n} = 1;
             }
@@ -923,6 +955,8 @@ foreach my $lib (@modules_to_sync) {
         foreach my $subdir (@subdirs) {
             if (opendir DIR, $subdir) {
                 foreach my $t (sort { $b cmp $a } readdir(DIR)) {
+                    next if ($t =~ /\.pri$/);
+                    next if ($t =~ /^qt[a-z0-9]+-config(_p)?\.h$/);
                     my $file = "$subdir/$t";
                     if(-d $file) {
                         push @subdirs, $file unless($t eq "." || $t eq "..");
@@ -972,17 +1006,21 @@ foreach my $lib (@modules_to_sync) {
 
             #calc files and "copy" them
             foreach my $subdir (@subdirs) {
-                my @headers = findFiles($subdir, "^[-a-z0-9_]*\\.h\$" , 0);
+                my @headers = findFiles($subdir, "^[-a-z0-9_]*\\.h\$");
+                @headers = grep(!/^qt[a-z0-9]+-config(_p)?\.h$/, @headers);
                 if (defined $inject_headers{$subdir}) {
                     foreach my $if (@{$inject_headers{$subdir}}) {
-                        @headers = grep(!/^\Q$if\E$/, @headers); #in case we configure'd previously
+                        my $cif = $if;
+                        $cif =~ s/^\^//;
+                        @headers = grep(!/^\Q$cif\E$/, @headers); #in case we configure'd previously
                         push @headers, "*".$if;
                     }
                 }
                 my $header_dirname = "";
                 foreach my $header (@headers) {
                     my $shadow = ($header =~ s/^\*//);
-                    $header = 0 if($header =~ /^ui_.*.h/);
+                    my $no_stamp = $shadow && ($header =~ s/^\^//);
+                    $header = 0 if ($header =~ /^ui_.*\.h$/);
                     foreach (@ignore_headers) {
                         $header = 0 if($header eq $_);
                     }
@@ -994,24 +1032,26 @@ foreach my $lib (@modules_to_sync) {
                         if(isQpaHeader($public_header)) {
                             $public_header = 0;
                             $qpa_header = 1;
-                        } elsif($allheadersprivate || $thisprivate || $public_header =~ /_p.h$/ || $public_header =~ /_pch.h$/) {
+                        } elsif ($allheadersprivate || $thisprivate || isPrivateHeader($public_header)) {
                             $public_header = 0;
-                        } else {
-                            foreach (@ignore_for_master_contents) {
-                                $public_header = 0 if($header eq $_);
-                            }
                         }
 
                         my $clean_header;
-                        my $iheader = $subdir . "/" . $header;
-                        $iheader =~ s/^\Q$basedir\E/$out_basedir/ if ($shadow);
+                        my $requires;
+                        my $iheader_src = $subdir . "/" . $header;
+                        my $iheader = $iheader_src;
+                        $iheader =~ s/^\Q$basedir\E/$build_basedir/ if ($shadow);
                         if ($check_includes) {
                             # We need both $public_header and $private_header because QPA headers count as neither
                             my $private_header = !$public_header && !$qpa_header
                                 && $header =~ /_p\.h$/ && $subdir !~ /3rdparty/;
                             check_header($lib, $header, $iheader, $public_header, $private_header);
                         }
-                        my @classes = $public_header && (!$minimal && $is_qt) ? classNames($iheader, \$clean_header) : ();
+                        my @classes = ();
+                        push @classes, classNames($iheader, \$clean_header, \$requires)
+                            if (!$shadow && $public_header && !$minimal && $is_qt);
+                        my $classname = $classnames{$header};
+                        push @classes, split(/,/, $classname) if ($classname);
                         if($showonly) {
                             print "$header [$lib]\n";
                             foreach(@classes) {
@@ -1034,42 +1074,45 @@ foreach my $lib (@modules_to_sync) {
 
                                     $header_copies++ if (!$shadow && syncHeader($lib, "$out_basedir/include/$lib/$class", "$out_basedir/include/$lib/$header", 0, $ts));
                                 }
-                            } elsif ($create_private_headers && !$qpa_header) {
+                            } elsif (!$qpa_header) {
                                 $oheader = "$out_basedir/include/$lib/$module_version/$lib/private/$header";
-                            } elsif ($create_private_headers) {
+                            } else {
                                 $oheader = "$out_basedir/include/$lib/$module_version/$lib/qpa/$header";
                             }
                             $header_copies++ if (!$shadow && syncHeader($lib, $oheader, $iheader, $copy_headers, $ts));
 
-                            my $pri_install_iheader = fixPaths($iheader, $dir);
+                            my $pri_install_iheader = fixPaths($iheader_src, $dir);
                             my $injection = "";
-                            if($public_header) {
-                                #put it into the master file
-                                $master_contents .= "#include \"$public_header\"\n" if (!$shadow && shouldMasterInclude($iheader));
-
-                                #deal with the install directives
+                            if ($public_header) {
                                 foreach my $class (@classes) {
                                     # Strip namespaces:
                                     $class =~ s/^.*:://;
 #                                   if ($class =~ m/::/) {
 #                                       $class =~ s,::,/,g;
 #                                   }
-                                    my $class_header = fixPaths("$out_basedir/include/$lib/$class", $dir) . " ";
-                                    $pri_install_classes .= $class_header
-                                                                unless($pri_install_classes =~ $class_header);
+                                    my $class_header = "$class ";
+                                    $pri_install_gfiles .= $class_header
+                                        unless ($shadow || $pri_install_gfiles =~ m/\b$class_header/);
                                     $injection .= ":$class";
                                 }
-                                $pri_install_files.= "$pri_install_iheader ";;
-                                $pri_clean_files .= "$pri_install_iheader " if ($clean_header);
+
+                                if (!$shadow) {
+                                    # put it into the master file
+                                    $master_contents{$public_header} = $requires if (shouldMasterInclude($iheader));
+
+                                    # deal with the install directives
+                                    $pri_install_files .= "$pri_install_iheader ";
+                                    $pri_clean_files .= "$pri_install_iheader".($requires ? ":".$requires : "")." " if ($clean_header);
+                                }
                             }
                             elsif ($qpa_header) {
                                 $pri_install_qpafiles.= "$pri_install_iheader ";;
                             }
-                            else {
+                            elsif (!$shadow) {
                                 $pri_install_pfiles.= "$pri_install_iheader ";;
                             }
-                            $pri_injections .= fixPaths($iheader, "$out_basedir/include/$lib")
-                                               .":".fixPaths($oheader, "$out_basedir/include/$lib")
+                            $pri_injections .= fixPaths($iheader, $build_basedir)
+                                               .":".($no_stamp ? "^" : "").fixPaths($oheader, "$out_basedir/include/$lib")
                                                .$injection." " if ($shadow);
                         }
 
@@ -1099,8 +1142,17 @@ foreach my $lib (@modules_to_sync) {
         }
     }
 
-    # close the master include:
-    $master_contents .=
+    # populate the master include:
+    my $master_contents =
+        "#ifndef QT_".$libcapitals."_MODULE_H\n" .
+        "#define QT_".$libcapitals."_MODULE_H\n" .
+        "#include <$lib/${lib}Depends>\n" .
+        join("", map {
+            my $rq = $master_contents{$_};
+            ($rq ? "#if QT_CONFIG($rq)\n" : "") .
+            "#include \"$_\"\n" .
+            ($rq ? "#endif\n" : "")
+        } sort globosort keys %master_contents) .
         "#include \"".lc($lib)."version.h\"\n" .
         "#endif\n";
 
@@ -1108,16 +1160,10 @@ foreach my $lib (@modules_to_sync) {
         # create deprecated headers
         my $first = 1;
         while (my ($header, $include) = each %{$deprecatedheaders{$lib}}) {
-            my $public_header = 0;
-            $public_header = 1 unless ($allheadersprivate || ($header =~ /_p\.h$/));
-            next unless ($public_header || $create_private_headers);
-
-            my $header_path = "$out_basedir/include/$lib/";
-            unless ($public_header) {
-                $header_path .= "$module_version/$lib/private/";
-            }
-            $header_path .= "$header";
+            die "Attempting unsupported aliasing of private header $header.\n"
+                if ($allheadersprivate || ($header =~ /_p\.h$/));
 
+            my $header_path = "$out_basedir/include/$lib/$header";
             unless (-e $header_path) {
                 my $guard = "DEPRECATED_HEADER_" . $lib . "_" . $header;
                 $guard =~ s/([^a-zA-Z0-9_])/_/g;
@@ -1125,26 +1171,19 @@ foreach my $lib (@modules_to_sync) {
                 my $header_dir = dirname($header_path);
                 make_path($header_dir, $lib, $verbose_level);
 
+                my $warning = "Header <$lib/$header> is deprecated. Please include <$include> instead.";
                 my $hdrcont =
                     "#ifndef $guard\n" .
-                    "#define $guard\n";
-                my $warning = "Header <$lib/";
-                $warning .= "private/" unless ($public_header);
-                $warning .= "$header> is deprecated. Please include <$include> instead.";
-                $hdrcont .=
+                    "#define $guard\n" .
                     "#if defined(__GNUC__)\n" .
                     "#  warning $warning\n" .
                     "#elif defined(_MSC_VER)\n" .
                     "#  pragma message (\"$warning\")\n" .
                     "#endif\n" .
-                    "#include <$include>\n";
-                if ($public_header) {
-                    $hdrcont .=
-                        "#if 0\n" .
-                        "#pragma qt_no_master_include\n" .
-                        "#endif\n";
-                }
-                $hdrcont .=
+                    "#include <$include>\n" .
+                    "#if 0\n" .
+                    "#pragma qt_no_master_include\n" .
+                    "#endif\n" .
                     "#endif\n";
                 if (writeFile($header_path, $hdrcont)) {
                     if ($verbose_level < 3) {
@@ -1158,12 +1197,7 @@ foreach my $lib (@modules_to_sync) {
                 }
             }
 
-            my $addendum = fixPaths($header_path, $dir) . " ";
-            if ($public_header) {
-                $pri_install_files .=  $addendum;
-            } else {
-                $pri_install_pfiles .=  $addendum;
-            }
+            $pri_install_gfiles .= "$header ";
         }
         if ($verbose_level < 3) {
             print " }\n" unless ($first);
@@ -1173,8 +1207,7 @@ foreach my $lib (@modules_to_sync) {
         my $vheader = "$out_basedir/include/$lib/".lc($lib)."version.h";
         my $VHeader = "$out_basedir/include/$lib/${lib}Version";
         syncHeader($lib, $VHeader, $vheader, 0);
-        $pri_install_files .= fixPaths($vheader, $dir) . " ";
-        $pri_install_classes .= fixPaths($VHeader, $dir) . " ";
+        $pri_install_gfiles .= lc($lib)."version.h ${lib}Version ";
         my @versions = split(/\./, $module_version);
         my $modulehexstring = sprintf("0x%02X%02X%02X", $versions[0], $versions[1], $versions[2]);
         my $vhdrcont =
@@ -1190,7 +1223,7 @@ foreach my $lib (@modules_to_sync) {
         writeFile($vheader, $vhdrcont, $lib, "version header");
 
         my $master_include = "$out_basedir/include/$lib/$lib";
-        $pri_install_files .= fixPaths($master_include, $dir) . " ";
+        $pri_install_gfiles .= "$lib ";
         writeFile($master_include, $master_contents, $lib, "master header");
     }
 
@@ -1198,7 +1231,7 @@ foreach my $lib (@modules_to_sync) {
         #handle the headers.pri for each module
         my $headers_pri_contents = "";
         $headers_pri_contents .= "SYNCQT.HEADER_FILES = $pri_install_files\n";
-        $headers_pri_contents .= "SYNCQT.HEADER_CLASSES = $pri_install_classes\n";
+        $headers_pri_contents .= "SYNCQT.GENERATED_HEADER_FILES = $pri_install_gfiles\n";
         $headers_pri_contents .= "SYNCQT.PRIVATE_HEADER_FILES = $pri_install_pfiles\n";
         $headers_pri_contents .= "SYNCQT.QPA_HEADER_FILES = $pri_install_qpafiles\n";
         $headers_pri_contents .= "SYNCQT.CLEAN_HEADER_FILES = $pri_clean_files\n";
